var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { sendMessageLogToSlack } from "./notify";
var LEVEL_DEBUG = {
    INFO: "info",
    WARN: "warn",
    ERROR: "error",
};
export var request = my.request;
var Logger = /** @class */ (function () {
    function Logger() {
        var _this = this;
        this.isSendToSlack = false;
        this.isApplyConsoleLog = false;
        this.slackUrl = "";
        this.appName = "";
        this.whitelist = ["*"];
        this.trackingLog = [];
        this.trackingRequest = [];
        this.getDebug = function () {
            var _getConfig = _this.getConfig.bind(_this);
            return {
                log: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    addLog(args, LEVEL_DEBUG.INFO, _getConfig());
                },
                warn: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    addLog(args, LEVEL_DEBUG.WARN, _getConfig());
                },
                error: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    addLog(args, LEVEL_DEBUG.ERROR, _getConfig());
                },
            };
        };
        this.getRequest = function (options) {
            var _trackingRequest = _this.trackingRequest;
            var response, resHeaders, isError = false;
            var success = options.success, fail = options.fail, complete = options.complete, url = options.url, headers = options.headers, method = options.method, data = options.data;
            request(__assign(__assign({}, options), { includeHeader: true, success: function (res) {
                    success && success(res.data);
                    response = res.data;
                    resHeaders = res.headers.map ? res.headers.map : res.headers;
                }, fail: function (error) {
                    fail && fail(error);
                    isError = true;
                    var cUrlHeader = [];
                    for (var key in headers) {
                        cUrlHeader.push("--header '".concat(key, ": ").concat(headers[key], "'"));
                    }
                    var cUrl = "curl --location --request ".concat(method, " '").concat(url, "' \\\n").concat(cUrlHeader.join(" \\\n")).concat(data ? " \\\n--data-raw '".concat(JSON.stringify(data), "'") : "", "\n      ");
                    my.debug.error("Fail request API ".concat(url, ": ").concat(error, "\n").concat(cUrl));
                    response = error;
                }, complete: function (res) {
                    complete && complete(res);
                    var requestName = url
                        .replace(/^(http:|https:\/\/)?(www\\.)?/i, "")
                        .split("/");
                    _trackingRequest.push({
                        general: {
                            url: url,
                            method: method || "GET",
                            time: new Date().toString(),
                        },
                        data: data,
                        headers: headers,
                        resHeaders: resHeaders,
                        requestInfo: {
                            title: requestName.pop(),
                            subTitle: requestName.join("/"),
                        },
                        response: JSON.stringify(response, null, 2),
                        isError: isError,
                        time: new Date(),
                    });
                } }));
        };
        this.debug = this.getDebug();
        this.request = this.getRequest;
    }
    Logger.prototype.init = function (_a) {
        var isOverwriteLog = _a.isOverwriteLog, isOverwriteRequest = _a.isOverwriteRequest, isApplyConsoleLog = _a.isApplyConsoleLog, config = _a.config;
        this.setConfig(__assign(__assign({}, config), { isApplyConsoleLog: isApplyConsoleLog || false }));
        if (isOverwriteLog) {
            my.debug = this.debug;
        }
        if (isOverwriteRequest) {
            my.request = this.request;
        }
    };
    Logger.prototype.setConfig = function (_a) {
        var whitelist = _a.whitelist, isSendToSlack = _a.isSendToSlack, slackUrl = _a.slackUrl, appName = _a.appName, isApplyConsoleLog = _a.isApplyConsoleLog;
        if (whitelist)
            this.whitelist = whitelist;
        if (isSendToSlack)
            this.isSendToSlack = isSendToSlack;
        if (slackUrl)
            this.slackUrl = slackUrl;
        if (appName)
            this.appName = appName;
        if (isApplyConsoleLog)
            this.isApplyConsoleLog = isApplyConsoleLog;
    };
    Logger.prototype.getConfig = function () {
        var _a = this, appName = _a.appName, isSendToSlack = _a.isSendToSlack, slackUrl = _a.slackUrl, trackingLog = _a.trackingLog, isApplyConsoleLog = _a.isApplyConsoleLog;
        return { appName: appName, isSendToSlack: isSendToSlack, slackUrl: slackUrl, trackingLog: trackingLog, isApplyConsoleLog: isApplyConsoleLog };
    };
    return Logger;
}());
export default new Logger();
var stringifyDebug = function (items) {
    var _a = items.reduce(function (_a, item) {
        var detail = _a.detail, message = _a.message;
        message.push(JSON.stringify(item));
        detail.push(JSON.stringify(item, null, 2));
        return { detail: detail, message: message };
    }, { detail: [], message: [] }), detail = _a.detail, message = _a.message;
    return {
        message: message.join(", "),
        detail: detail.join("\n\n"),
    };
};
function addLog(msgs, type, config) {
    var _a = stringifyDebug(msgs), detail = _a.detail, message = _a.message;
    var time = new Date();
    var trackingLog = config.trackingLog, isSendToSlack = config.isSendToSlack, appName = config.appName, slackUrl = config.slackUrl, isApplyConsoleLog = config.isApplyConsoleLog;
    trackingLog.push({ message: message, detail: detail, type: type, time: time });
    if (isSendToSlack && type === LEVEL_DEBUG.ERROR) {
        sendMessageLogToSlack({ message: msgs, type: type }, appName, slackUrl);
    }
    if (isApplyConsoleLog) {
        console.log.apply(console, msgs);
    }
}
